import { Logger, type LoggerOptions } from "@sfroment/logger";
import type { ACL } from "./acl/interface.js";
import { type FinalityConfig, FinalityStore } from "./finality/index.js";
import { HashGraph, type ResolveConflictsType, type Vertex } from "./hashgraph/index.js";
import { type DRP, type DRPObjectCallback, type DRPPublicCredential, DrpType } from "./interface.js";
import * as ObjectPb from "./proto/drp/object/v1/object_pb.js";
export * as ObjectPb from "./proto/drp/object/v1/object_pb.js";
export * from "./utils/serializer.js";
export * from "./acl/index.js";
export * from "./hashgraph/index.js";
export * from "./acl/interface.js";
export * from "./interface.js";
export interface DRPObjectConfig {
    log_config?: LoggerOptions;
    finality_config?: FinalityConfig;
}
export declare let log: Logger;
export declare class DRPObject implements ObjectPb.DRPObjectBase {
    id: string;
    peerId: string;
    vertices: ObjectPb.Vertex[];
    acl?: ProxyHandler<ACL>;
    drp?: ProxyHandler<DRP>;
    hashGraph: HashGraph;
    drpStates: Map<string, ObjectPb.DRPState>;
    aclStates: Map<string, ObjectPb.DRPState>;
    originalDRP?: DRP;
    originalObjectACL?: ACL;
    finalityStore: FinalityStore;
    subscriptions: DRPObjectCallback[];
    constructor(options: {
        peerId: string;
        publicCredential?: DRPPublicCredential;
        acl?: ACL;
        drp?: DRP;
        id?: string;
        config?: DRPObjectConfig;
    });
    private _initLocalDrpInstance;
    private _initNonLocalDrpInstance;
    static createObject(options: {
        peerId: string;
        id?: string;
        drp?: DRP;
    }): DRPObject;
    resolveConflicts(vertices: Vertex[]): ResolveConflictsType;
    proxyDRPHandler(vertexType: DrpType): ProxyHandler<object>;
    private callFn;
    merge(vertices: Vertex[]): [merged: boolean, missing: string[]];
    subscribe(callback: DRPObjectCallback): void;
    private _notify;
    private _initializeFinalityState;
    private _checkWriterPermission;
    private _applyOperation;
    private _computeDRP;
    private _computeObjectACL;
    private computeLCA;
    private _getDRPState;
    private _computeDRPState;
    private _computeObjectACLState;
    private _setState;
    private _setObjectACLState;
    private _setDRPState;
    private _updateDRPState;
    private _updateObjectACLState;
    private _setRootStates;
}
//# sourceMappingURL=index.d.ts.map